@@ -1,6 +1,6 @@
 #!/bin/sh
 PKGS_DIR="/sabayon/remaster/pkgs"
-CHROOT_PKGS_DIR="${CHROOT_DIR}/var/lib/entropy/packages"
+CHROOT_PKGS_DIR="${CHROOT_DIR}/var/lib/entropy/client/packages"
 
 echo "Merging back packages"
 cp "${CHROOT_PKGS_DIR}"/* "${PKGS_DIR}"/ -Ra
