@@ -1,6 +1,6 @@
 #!/bin/sh
 PKGS_DIR="/sabayon/remaster/pkgs"
-CHROOT_PKGS_DIR="${CHROOT_DIR}/var/lib/entropy/packages"
+CHROOT_PKGS_DIR="${CHROOT_DIR}/var/lib/entropy/client/packages"
 
 [[ ! -d "${PKGS_DIR}" ]] && mkdir -p "${PKGS_DIR}"
 [[ ! -d "${CHROOT_PKGS_DIR}" ]] && mkdir -p "${CHROOT_PKGS_DIR}"
