@@ -18,6 +18,9 @@
 
 import os
 import shutil
+import tempfile
+import glob
+
 from molecule.compat import get_stringtype
 from molecule.i18n import _
 from molecule.output import red, brown, blue, green, purple, darkgreen, \
@@ -29,49 +32,62 @@ class BuiltinHandlerMixin:
     """
     This class contains code in common between built-in handler classes.
     """
-    def _run_error_script(self, source_chroot_dir, chroot_dir, cdroot_dir):
+    def _run_error_script(self, source_chroot_dir, chroot_dir, cdroot_dir,
+        env = None):
+
         error_script = self.metadata.get('error_script')
         if error_script:
+
+            if env is None:
+                env = os.environ.copy()
+            else:
+                env = env.copy()
+
             if source_chroot_dir:
-                os.environ['SOURCE_CHROOT_DIR'] = source_chroot_dir
+                env['SOURCE_CHROOT_DIR'] = source_chroot_dir
             if chroot_dir:
-                os.environ['CHROOT_DIR'] = chroot_dir
+                env['CHROOT_DIR'] = chroot_dir
             if cdroot_dir:
-                os.environ['CDROOT_DIR'] = cdroot_dir
+                env['CDROOT_DIR'] = cdroot_dir
             self._output.output("[%s|%s] %s: %s" % (
                     blue("BuiltinHandler"), darkred(self.spec_name),
-                    _("spawning"), error_script,
+                    _("spawning"), " ".join(error_script),
                 )
             )
-            molecule.utils.exec_cmd(error_script)
-            for env_key in ("SOURCE_CHROOT_DIR", "CHROOT_DIR", "CDROOT_DIR",):
-                try:
-                    del os.environ[env_key]
-                except KeyError:
-                    continue
+            molecule.utils.exec_cmd(error_script, env = env)
 
     def _exec_inner_script(self, exec_script, dest_chroot):
 
-        while True:
-            tmp_dir = os.path.join(dest_chroot,
-                str(molecule.utils.get_random_number()))
-            if not os.path.lexists(tmp_dir):
-                break
+        source_exec = exec_script[0]
+        with open(source_exec, "rb") as f_src:
+            tmp_fd, tmp_exec = tempfile.mkstemp(dir = dest_chroot,
+                prefix = "molecule_inner")
+            f_dst = os.fdopen(tmp_fd, "wb")
+            try:
+                shutil.copyfileobj(f_src, f_dst)
+            finally:
+                f_dst.flush()
+                f_dst.close()
+            shutil.copystat(source_exec, tmp_exec)
+            os.chmod(tmp_exec, 0o744)
 
-        os.makedirs(tmp_dir)
-        tmp_exec = os.path.join(tmp_dir, "inner_exec")
-        shutil.copy2(exec_script[0], tmp_exec)
-        os.chmod(tmp_exec, 0o755)
-        dest_exec = tmp_exec[len(dest_chroot):]
+        dest_exec = os.path.basename(tmp_exec)
         if not dest_exec.startswith("/"):
             dest_exec = "/%s" % (dest_exec,)
 
-        rc = molecule.utils.exec_chroot_cmd([dest_exec] + exec_script[1:],
-            dest_chroot, self.metadata.get('prechroot', []))
-        os.remove(tmp_exec)
-        os.rmdir(tmp_dir)
+        rc = 0
+        try:
+            rc = molecule.utils.exec_chroot_cmd([dest_exec] + exec_script[1:],
+                dest_chroot, pre_chroot = self.metadata.get('prechroot', []))
+        except:
+            # kill all the pids inside chroot, no matter what just happened
+            molecule.utils.kill_chroot_pids(dest_chroot, sleep = True)
+            raise
+        finally:
+            os.remove(tmp_exec)
 
         if rc != 0:
+            molecule.utils.kill_chroot_pids(dest_chroot, sleep = True)
             self._output.output("[%s|%s] %s: %s" % (
                     blue("BuiltinHandler"), darkred(self.spec_name),
                     _("inner chroot hook failed"), rc,
@@ -93,6 +109,7 @@ class MirrorHandler(GenericExecutionStep, BuiltinHandlerMixin):
         )
         if not os.path.isdir(self.dest_dir):
             os.makedirs(self.dest_dir, 0o755)
+        return 0
 
     def pre_run(self):
         self._output.output("[%s|%s] %s" % (
@@ -152,10 +169,11 @@ class MirrorHandler(GenericExecutionStep, BuiltinHandlerMixin):
         args = [self._config['mirror_syncer']]
         args.extend(self._config['mirror_syncer_builtin_args'])
         args.extend(self.metadata.get('extra_rsync_parameters', []))
-        args.extend([self.source_dir+"/*", self.dest_dir])
+        args.append(self.source_dir + "/")
+        args.append(self.dest_dir + "/")
         self._output.output("[%s|%s] %s: %s" % (
                 blue("MirrorHandler"), darkred(self.spec_name),
-                _("spawning"), args,
+                _("spawning"), " ".join(args),
             )
         )
         rc = molecule.utils.exec_cmd(args)
@@ -185,6 +203,7 @@ class ChrootHandler(GenericExecutionStep, BuiltinHandlerMixin):
             self.metadata['destination_chroot'], "chroot",
             os.path.basename(self.source_dir)
         )
+        return 0
 
     def pre_run(self):
         self._output.output("[%s|%s] %s" % (
@@ -196,13 +215,14 @@ class ChrootHandler(GenericExecutionStep, BuiltinHandlerMixin):
         # run outer chroot script
         exec_script = self.metadata.get('outer_chroot_script')
         if exec_script:
-            os.environ['CHROOT_DIR'] = self.source_dir
+            env = os.environ.copy()
+            env['CHROOT_DIR'] = self.source_dir
             self._output.output("[%s|%s] %s: %s" % (
                     blue("ChrootHandler"), darkred(self.spec_name),
-                    _("spawning"), exec_script,
+                    _("spawning"), " ".join(exec_script),
                 )
             )
-            rc = molecule.utils.exec_cmd(exec_script)
+            rc = molecule.utils.exec_cmd(exec_script, env = env)
             if rc != 0:
                 self._output.output("[%s|%s] %s: %s" % (
                         blue("ChrootHandler"), darkred(self.spec_name),
@@ -294,13 +314,14 @@ class ChrootHandler(GenericExecutionStep, BuiltinHandlerMixin):
         # run outer chroot script after
         exec_script = self.metadata.get('outer_chroot_script_after')
         if exec_script:
-            os.environ['CHROOT_DIR'] = self.source_dir
+            env = os.environ.copy()
+            env['CHROOT_DIR'] = self.source_dir
             self._output.output("[%s|%s] %s: %s" % (
                     blue("ChrootHandler"), darkred(self.spec_name),
-                    _("spawning"), exec_script,
+                    _("spawning"), " ".join(exec_script),
                 )
             )
-            rc = molecule.utils.exec_cmd(exec_script)
+            rc = molecule.utils.exec_cmd(exec_script, env = env)
             if rc != 0:
                 self._output.output("[%s|%s] %s: %s" % (
                         blue("ChrootHandler"), darkred(self.spec_name),
@@ -363,6 +384,7 @@ class CdrootHandler(GenericExecutionStep, BuiltinHandlerMixin):
             molecule.utils.empty_dir(self.dest_root)
         if not os.path.isdir(self.dest_root):
             os.makedirs(self.dest_root, 0o755)
+        return 0
 
     def pre_run(self):
         self._output.output("[%s|%s] %s" % (
@@ -414,7 +436,7 @@ class CdrootHandler(GenericExecutionStep, BuiltinHandlerMixin):
         args.extend(self.metadata.get('extra_mksquashfs_parameters', []))
         self._output.output("[%s|%s] %s: %s" % (
                 blue("CdrootHandler"), darkred(self.spec_name),
-                _("spawning"), args,
+                _("spawning"), " ".join(args),
             )
         )
         rc = molecule.utils.exec_cmd(args)
@@ -440,28 +462,34 @@ class CdrootHandler(GenericExecutionStep, BuiltinHandlerMixin):
                     _("merging livecd root"), merge_dir,)
                 )
                 import stat
-                content = os.listdir(merge_dir)
-                for mypath in content:
-                    mysource = os.path.join(merge_dir, mypath)
-                    mydest = os.path.join(self.dest_root, mypath)
-                    copystat = False
+                try:
+                    content = os.listdir(merge_dir)
+                    for mypath in content:
+                        mysource = os.path.join(merge_dir, mypath)
+                        mydest = os.path.join(self.dest_root, mypath)
+                        copystat = False
 
-                    if os.path.islink(mysource):
-                        tolink = os.readlink(mysource)
-                        os.symlink(tolink, mydest)
-                    elif os.path.isfile(mysource) or os.path.islink(mysource):
-                        copystat = True
-                        shutil.copy2(mysource, mydest)
-                    elif os.path.isdir(mysource):
-                        copystat = True
-                        shutil.copytree(mysource, mydest)
-
-                    if copystat:
-                        user = os.stat(mysource)[stat.ST_UID]
-                        group = os.stat(mysource)[stat.ST_GID]
-                        os.chown(mydest, user, group)
-                        shutil.copystat(mysource, mydest)
+                        if os.path.islink(mysource):
+                            tolink = os.readlink(mysource)
+                            os.symlink(tolink, mydest)
+                        elif os.path.isfile(mysource) or os.path.islink(mysource):
+                            copystat = True
+                            shutil.copy2(mysource, mydest)
+                        elif os.path.isdir(mysource):
+                            copystat = True
+                            shutil.copytree(mysource, mydest)
 
+                        if copystat:
+                            user = os.stat(mysource)[stat.ST_UID]
+                            group = os.stat(mysource)[stat.ST_GID]
+                            os.chown(mydest, user, group)
+                            shutil.copystat(mysource, mydest)
+                except OSError as err:
+                    self._output.output("[%s|%s] %s: %s" % (
+                        blue("CdrootHandler"), darkred(self.spec_name),
+                        _("cannot merge livecd root, improper usage or system error"), err,)
+                    )
+                    return 1
         return 0
 
 class IsoHandler(GenericExecutionStep, BuiltinHandlerMixin):
@@ -497,6 +525,7 @@ class IsoHandler(GenericExecutionStep, BuiltinHandlerMixin):
             self.metadata['destination_chroot'], "chroot",
             os.path.basename(self.source_chroot)
         )
+        return 0
 
     def pre_run(self):
         self._output.output("[%s|%s] %s" % (
@@ -508,17 +537,18 @@ class IsoHandler(GenericExecutionStep, BuiltinHandlerMixin):
         # run pre iso script
         exec_script = self.metadata.get('pre_iso_script')
         if exec_script:
-            os.environ['SOURCE_CHROOT_DIR'] = self.source_chroot
-            os.environ['CHROOT_DIR'] = self.chroot_dir
-            os.environ['CDROOT_DIR'] = self.source_path
-            os.environ['ISO_PATH'] = self.dest_iso
-            os.environ['ISO_CHECKSUM_PATH'] = self.dest_iso + IsoHandler.MD5_EXT
+            env = os.environ.copy()
+            env['SOURCE_CHROOT_DIR'] = self.source_chroot
+            env['CHROOT_DIR'] = self.chroot_dir
+            env['CDROOT_DIR'] = self.source_path
+            env['ISO_PATH'] = self.dest_iso
+            env['ISO_CHECKSUM_PATH'] = self.dest_iso + IsoHandler.MD5_EXT
             self._output.output("[%s|%s] %s: %s" % (
                     blue("IsoHandler"), darkred(self.spec_name),
-                    _("spawning"), exec_script,
+                    _("spawning"), " ".join(exec_script),
                 )
             )
-            rc = molecule.utils.exec_cmd(exec_script)
+            rc = molecule.utils.exec_cmd(exec_script, env = env)
             if rc != 0:
                 self._output.output("[%s|%s] %s: %s" % (
                         blue("IsoHandler"), darkred(self.spec_name),
@@ -536,17 +566,18 @@ class IsoHandler(GenericExecutionStep, BuiltinHandlerMixin):
             )
         )
 
-        # run pre iso script
+        # run post iso script
         exec_script = self.metadata.get('post_iso_script')
         if exec_script:
-            os.environ['ISO_PATH'] = self.dest_iso
-            os.environ['ISO_CHECKSUM_PATH'] = self.dest_iso + IsoHandler.MD5_EXT
+            env = os.environ.copy()
+            env['ISO_PATH'] = self.dest_iso
+            env['ISO_CHECKSUM_PATH'] = self.dest_iso + IsoHandler.MD5_EXT
             self._output.output("[%s|%s] %s: %s" % (
                     blue("IsoHandler"), darkred(self.spec_name),
-                    _("spawning"), exec_script,
+                    _("spawning"), " ".join(exec_script),
                 )
             )
-            rc = molecule.utils.exec_cmd(exec_script)
+            rc = molecule.utils.exec_cmd(exec_script, env = env)
             if rc != 0:
                 self._output.output("[%s|%s] %s: %s" % (
                         blue("IsoHandler"), darkred(self.spec_name),
@@ -580,11 +611,11 @@ class IsoHandler(GenericExecutionStep, BuiltinHandlerMixin):
         args.extend(self._config['iso_builder_builtin_args'])
         args.extend(self.metadata.get('extra_mkisofs_parameters', []))
         if self.iso_title.strip():
-            args.extend(["-V", '"', self.iso_title[:30], '"'])
+            args.extend(["-V", self.iso_title[:32]])
         args.extend(['-o', self.dest_iso, self.source_path])
         self._output.output("[%s|%s] %s: %s" % (
                 blue("IsoHandler"), darkred(self.spec_name),
-                _("spawning"), args,
+                _("spawning"), " ".join(args),
             )
         )
         rc = molecule.utils.exec_cmd(args)
@@ -633,9 +664,13 @@ class LivecdSpec(GenericSpec):
 
     def parser_data_path(self):
         return {
+            'execution_strategy': {
+                'cb': self.ne_string,
+                've': self.ve_string_stripper,
+            },
             'prechroot': {
                 'cb': self.valid_exec_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'release_string': {
                 'cb': self.ne_string, # validation callback
@@ -663,7 +698,7 @@ class LivecdSpec(GenericSpec):
             },
             'extra_rsync_parameters': {
                 'cb': self.always_valid,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'merge_destination_chroot': {
                 'cb': self.valid_dir,
@@ -671,23 +706,23 @@ class LivecdSpec(GenericSpec):
             },
             'error_script': {
                 'cb': self.valid_exec_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'outer_chroot_script': {
                 'cb': self.valid_exec_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'inner_source_chroot_script': {
                 'cb': self.valid_path_string_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'inner_chroot_script': {
                 'cb': self.valid_path_string_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'outer_chroot_script_after': {
                 'cb': self.valid_exec_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'destination_livecd_root': {
                 'cb': self.valid_path_string,
@@ -699,19 +734,19 @@ class LivecdSpec(GenericSpec):
             },
             'extra_mksquashfs_parameters': {
                 'cb': self.always_valid,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'extra_mkisofs_parameters': {
                 'cb': self.always_valid,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'pre_iso_script': {
                 'cb': self.valid_exec_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'post_iso_script': {
                 'cb': self.valid_exec_first_list_item,
-                've': self.ve_string_splitter,
+                've': self.ve_command_splitter,
             },
             'destination_iso_directory': {
                 'cb': self.valid_dir,
